////////////////////////////////////////////////////////////////////////////////
//
//               Groundfire
//
////////////////////////////////////////////////////////////////////////////////
//
// Copyright (c) 2004, Tom Russell (tom@groundfire.net)
//
// This file is part of the Groundfire project, distributed under the MIT 
// license. See the file 'COPYING', included with this distribution, for a copy
// of the full MIT licence.
//
////////////////////////////////////////////////////////////////////////////////
//
//   File name : game.cc
//
//          By : Tom Russell
//
//        Date : 07-Sep-02
//
// Description : The main framework of the game. This module keeps track of the 
//               current state the game is in and also stores the list of 
//               entities currently active in the game.
//
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Includes
////////////////////////////////////////////////////////////////////////////////
#include "game.hh"

#include "tank.hh"
#include "mainmenu.hh"
#include "quitmenu.hh"
#include "optionmenu.hh"
#include "controllermenu.hh"
#include "setcontrolsmenu.hh"
#include "playermenu.hh"
#include "winnermenu.hh"
#include "scoremenu.hh"
#include "shopmenu.hh"
#include "sounds.hh"
#include "quake.hh"
#include "missile.hh"
#include "missileweapon.hh"
#include "shellweapon.hh"
#include "nukeweapon.hh"
#include "trail.hh"
#include "blast.hh"

#include <GL/glfw.h>
#include <stdlib.h>

////////////////////////////////////////////////////////////////////////////////
// Public Member Functions
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
//
// Function    : cGame
//
// Description : Constructor
//
////////////////////////////////////////////////////////////////////////////////
cGame::cGame
(
) 
        : _settings ("conf/options.ini"),
          // Create the interface object
          _interface (_settings.getInt ("Graphics", "ScreenWidth",  640),
                      _settings.getInt ("Graphics", "ScreenHeight", 480),
                      _settings.getInt ("Graphics", "Fullscreen",   0))
{
    // load the textures and sounds
    loadResources ();

    // These variables are used to determine the time elapsed between frames 
    // and thus, the framerate.
    _frameMeasureCount = 20;
    _frameMeasureTime  = 0.0f;
    _currentFPS        = 0.0f;

    // Should we display the frame rate during the game?
    _showFPS = _settings.getInt ("Graphics", "ShowFPS", 0);

    // Read Settings for classes
    cShellWeapon::readSettings (_settings);
    cNukeWeapon::readSettings (_settings);
    cMissile::readSettings (_settings);
    cQuake::readSettings (_settings);
    cMissileWeapon::readSettings (_settings);
    cTrail::readSettings (_settings);
    cBlast::readSettings (_settings);

    // Clear the tanks array 
    _numberOfTanks = 0;

    for (int i = 0; i < 8; i++)
    {
        _tank[i] = NULL;
    }

    _landscape   = NULL; 

    // Initialise the state of the game. We start on the main menu
    _gameState   = MAIN_MENU;
    _currentMenu = new cMainMenu (this);
    _interface.enableMouse (true);

    // Create the controls object to deal with the mapping between the game 
    // controls and the controller devices.
    _controls     = new cControls (&_interface);
    _controlsFile = new cControlsFile (_controls, "conf/controls.ini");
    _controlsFile->readFile ();

/*
    // This Code is here to allow testing of the various menus without having 
    // to go through the rest of the game first. It creates some tanks, sets up
    // some values and launches straight into one of the menus.
    {
        addTank (0);  
        _tank[0]->doPreRound ();
        _tank[0]->_score = 9009;
        _tank[0]->_money = 150;
        addTank (1);  
        _tank[1]->doPreRound ();
        _tank[1]->_score = 9001;
        _tank[1]->_money = 150;
        addTank (2);  
        _tank[2]->doPreRound ();
        _tank[2]->_score = 9001;
        _tank[2]->_money = 150;
        addTank (2);  
        _tank[3]->doPreRound ();
        _tank[3]->_score = 9000;
        _tank[3]->_money = 150;
        addTank (2);  
        _tank[4]->doPreRound ();
        _tank[4]->_score = 9001;
        _tank[4]->_money = 150;
        addTank (2);  
        _tank[5]->doPreRound ();
        _tank[5]->_score = 9001;
        _tank[5]->_money = 150;
        addTank (2);  
        _tank[6]->doPreRound ();
        _tank[6]->_score = 9001;
        _tank[6]->_money = 150;
        addTank (2);  
        _tank[7]->doPreRound ();
        _tank[7]->_score = 9001;
        _tank[7]->_money = 150;

        _tank[3]->_leader = true;
        _tank[0]->defeated (_tank[4]);
        _tank[0]->defeated (_tank[3]);
        _tank[5]->defeated (_tank[5]);

    }

    _gameState   = SHOP_MENU;
    _currentMenu = new cShopMenu (this);
*/

    // Finally, record the current time.
    _lastTick = glfwGetTime ();
}

////////////////////////////////////////////////////////////////////////////////
//
// Function    : ~cGame
//
// Description : Destructor
//
////////////////////////////////////////////////////////////////////////////////
cGame::~cGame
(
    void
) 
{
    if (_gameState == ROUND_STARTING ||
        _gameState == ROUND_IN_ACTION ||
        _gameState == ROUND_FINISHING ||
        _gameState == PAUSE_MENU)
    {
        endRound ();        
    }

    removeAllTanks ();

    if (_controls)
    {
        delete _controls;
    }

    if (_controlsFile)
    {
        delete _controlsFile;
    }

    if (_currentMenu) 
    {
        delete _currentMenu;
    }
    
    if (_font) 
    {
        delete _font;
    }

#ifndef NOSOUND
    if (_sound) 
    {
        delete _sound;
    }
#endif

}

////////////////////////////////////////////////////////////////////////////////
//
// Function    : loadResources
//
// Description : Load the external resources used by the game.
//
////////////////////////////////////////////////////////////////////////////////
void
cGame::loadResources
(
)
{
    // specify the number of textures
    _interface.defineTextures (10);

    // Load each of the textures used by the game
    loadTexture ("data/blast.tga",       0);
    loadTexture ("data/trail.tga",       1);
    loadTexture ("data/exhaust.tga",     2);
    loadTexture ("data/damage.tga",      4);
    loadTexture ("data/smoke.tga",       5);
    loadTexture ("data/menuback.tga",    6);
    loadTexture ("data/weaponicons.tga", 7);
    loadTexture ("data/arrow.tga",       8);
    loadTexture ("data/logo.tga",        9);

    // Create the font object used to render the text to the screen.
    _font = new cFont (&_interface, 3);

#ifndef NOSOUND
    // Initialise the sound system (openAL)
    _sound = new cSound (8);

    // Load all the Wav files used by the game
    _sound->loadSound (0, "data/fireshell.wav");
    _sound->loadSound (1, "data/shelldeath.wav");
    _sound->loadSound (2, "data/quake.wav");
    _sound->loadSound (3, "data/jumpjets.wav");
    _sound->loadSound (4, "data/missile.wav");
    _sound->loadSound (5, "data/launchmissile.wav");
    _sound->loadSound (6, "data/missiledeath.wav");
    _sound->loadSound (7, "data/nuke.wav");

#endif
}


////////////////////////////////////////////////////////////////////////////////
//
// Function    : loopOnce
//
// Description : Does one iteration of the game (i.e. Update & Draw). Returns 
//               'false' when the program is exiting.
//
////////////////////////////////////////////////////////////////////////////////
bool
cGame::loopOnce
(
    void
)
{   
    // Calculate the time that has passed since the last iteration
    double currentTick = glfwGetTime ();
    double elapsedTime = currentTick - _lastTick;    
    _lastTick = currentTick;
    
    // Start the frame
    _interface.startDraw ();
    
    switch (_gameState)
    {       
    case MAIN_MENU:
    case OPTION_MENU:
    case CONTROLLERS_MENU:
    case SET_CONTROLS_MENU:
    case QUIT_MENU:
    case SELECT_PLAYERS_MENU:
    case SHOP_MENU:
    case ROUND_SCORE:
    case WINNER_MENU:
        // All the menu states are handled in their own function.
        menuLoop (elapsedTime);
        break;
        
    case ROUND_STARTING:
        gameLoop (elapsedTime);
        
        // While the round is starting, draw the round number in the centre of 
        // the screen.
        _font->setShadow (true);
        _font->setSize   (0.6f, 0.6f, 0.5f);
        _font->setColour (1.0f, 1.0f, 1.0f);
        _font->printCentredAt (0.0f,  0.5f, "Round %d", _currentRound);
        _font->printCentredAt (0.0f, -0.5f, "Get Ready");
        _font->setShadow (false);
        
        // Countdown until the round begins
        _stateCountdown -= elapsedTime;
        if (_stateCountdown < 0.0f) 
        {
            _gameState = ROUND_IN_ACTION;
        }
        break;
        
    case ROUND_FINISHING:
        // When all but one tank is dead (or ALL tanks are dead), we start 
        // counting down for the round to end.
        gameLoop (elapsedTime);
        _stateCountdown -= elapsedTime;
        if (_stateCountdown < 0.0f) 
        {
            // That's it, round over! Go to the scoring menu.
            endRound ();
            _gameState = ROUND_SCORE;
            _currentMenu = new cScoreMenu (this);
            _interface.enableMouse (true);
        }            
        break;
        
    case ROUND_IN_ACTION:
        gameLoop (elapsedTime);
        break;
        
    case PAUSE_MENU:
        // The pause menu is not currently implemented.
        break;
        
    case EXITED:
        // A special state that tells this loop to terminate the program.
        return (false);
        
    default:
        break;
    }
    
    // If we have been told to display the Frames per second, we should draw it
    // now in the bottom left corner.
    if (_showFPS)
    {
        // To get a better estimate of the average FPS, only update the FPS
        // every 20 frames.
        if (_frameMeasureCount == 0) 
        {
            // This is the 20th frame so calculate a new FPS
            _currentFPS = 20.0f / _frameMeasureTime;
            _frameMeasureTime  = 0.0f;
            _frameMeasureCount = 20;
        }
        else
        {
            // This is not the 20th frame so don't calculate a new FPS
            _frameMeasureTime += elapsedTime;
            _frameMeasureCount--;
        }
        
        _font->setSize   (0.3f, 0.3f, 0.25f);
        _font->setColour (0.5f, 1.0f, 0.2f);
        
        _font->printf (-10.0f, -7.3f, "%.1f FPS", _currentFPS);
    }
    
    // End the frame
    _interface.endDraw ();
    
    // 'true' means we haven't finished.
    return (true);
}

////////////////////////////////////////////////////////////////////////////////
//
// Function    : gameLoop
//
// Description : implements the part of the loop that occurs while a round is 
//               in progress. (i.e. updating the entities.)
//
////////////////////////////////////////////////////////////////////////////////
void
cGame::gameLoop
(
    double elapsedTime
)

{
    list<cEntity *>::iterator iterator;

    // First we update everything. Landscape first and then the entities.
    
    _landscape->update ((float)elapsedTime);

    for (iterator  = _entityList.begin ();
         iterator != _entityList.end ();)
    {
        // If an entities update function returns 'false', that entity has died
        // and should be removed from the list of entities. 
        if (!(*iterator)->update ((float)elapsedTime))
        {
            iterator = _entityList.erase (iterator);
        }
        else
        {
            ++iterator;
        }
    }

    // Now we draw everything. First the landscape and then all the entities.

    _landscape->draw ();
    
    for (iterator  = _entityList.begin ();
         iterator != _entityList.end ();
         iterator++)
    {
        (*iterator)->draw ();
    }
}

////////////////////////////////////////////////////////////////////////////////
//
// Function    : menuLoop
//
// Description : Updates and Draws the current menu. Also, handles the state
//               changes while in a menu.
//
////////////////////////////////////////////////////////////////////////////////
void
cGame::menuLoop
(
    double elapsedTime
)

{
    // update and draw the current menu.
    enumGameState changeState = _currentMenu->update (elapsedTime);

    _currentMenu->draw ();

    // 'CURRENT_STATE' means we haven't changed state, otherwise we need to
    // change the state.
    if (changeState != CURRENT_STATE)
    {
        // Delete the current menu and set the new state
        _gameState = changeState;
        delete _currentMenu;
        _currentMenu = NULL;

        // Create a new menu (or go to the game) depending on what the new state
        // is.
        switch (_gameState)
        {
        case MAIN_MENU:
            _currentMenu = new cMainMenu (this);
            break;

        case OPTION_MENU:
            _currentMenu = new cOptionMenu (this);
            break;

        case CONTROLLERS_MENU:
            _currentMenu = new cControllerMenu (this);
            break;

        case SET_CONTROLS_MENU:
            _currentMenu = new cSetControlsMenu (this, _activeController);
            break;

        case QUIT_MENU:
            _currentMenu = new cQuitMenu (this);
            break;

        case SELECT_PLAYERS_MENU:
            _currentRound = 0;
            _currentMenu = new cPlayerMenu (this);
            break;

        case SHOP_MENU:
            _currentMenu = new cShopMenu (this);
            break;

        case WINNER_MENU:
            _currentMenu = new cWinnerMenu (this);
            break;

        case ROUND_STARTING:
            // We are starting a new round. Disable the mouse, set the
            // countdown and increment the round number.
            _interface.enableMouse (false);
            _stateCountdown = 2.0f;
            _currentRound++;
            startRound ();
            break;

        case EXITED:
            // We are quiting the program.
            break;

        default:
            break;
        }
    }
}

////////////////////////////////////////////////////////////////////////////////
//
// Function    : startRound
//
// Description : Do any initialisations needed before the start of a new round.
//
////////////////////////////////////////////////////////////////////////////////
void
cGame::startRound
(
)
{
    // Create a new landscape for the round.
    _landscape = new cLandscape (&_settings, _lastTick);
    
    // Call the 'doPreRound' function for any entities that have survived from 
    // the previous round. Usually this means the tanks.
    list<cEntity *>::iterator iterator;

    for (iterator  = _entityList.begin ();
         iterator != _entityList.end ();
         iterator++)
    {
        (*iterator)->doPreRound ();
    }

    // Work out how many tanks will feature in this round.
    _numberOfTanksInPlay = 0;

    int tankOrder[8];

    for (int i = 0; i < _numberOfTanks; i++)
    {
        if (_tank[i]->alive ()) 
        {
            tankOrder[_numberOfTanksInPlay] = i;

            _numberOfTanksInPlay++;
        }
    }

    // Choose a random location on the landscape to position each tank. This 
    // makes it a bit fairer, because any tank can appear anywhere. The tanks 
    // are always equally spaced out.

    // This first part jiggles the order across the screen that the tanks will 
    // be positioned.
    for (int i = 0; i < 20; i++) 
    {
        int firstTank  = rand () % _numberOfTanksInPlay;
        int secondTank = rand () % _numberOfTanksInPlay;

        if (firstTank != secondTank)
        {
            // Swap the tanks over. Yep, this does do a swap in case you were 
            // wondering.
            tankOrder[firstTank]  ^= tankOrder[secondTank];
            tankOrder[secondTank] ^= tankOrder[firstTank];
            tankOrder[firstTank]  ^= tankOrder[secondTank];
        }
    }

    // Finally, tell each tank where it is located.
    for (int i = 0; i < _numberOfTanksInPlay; i++) 
    {
        if (_tank[tankOrder[i]]->alive ())
        {

            _tank[tankOrder[i]]->setPositionOnGround (
                -10.0 + (10.0 / _numberOfTanksInPlay) 
                + (i * (20.0 / _numberOfTanksInPlay)));
        }
    }

    // Create an earthquake entity for this round.
    cQuake * earthquake = new cQuake (this);
    // insert at the front of the list so that it always gets updated first.
    _entityList.push_front (earthquake);
}

////////////////////////////////////////////////////////////////////////////////
//
// Function    : endRound
//
// Description : Do some tidying up at the end of a round.
//
////////////////////////////////////////////////////////////////////////////////
void
cGame::endRound
(
)
{
    // Call the doPostRound for all active entities, in most cases, this means 
    // they will self destruct.
    list<cEntity *>::iterator iterator;
    
    for (iterator  = _entityList.begin ();
         iterator != _entityList.end ();)
    {
        if (!(*iterator)->doPostRound ())
        {
            iterator = _entityList.erase (iterator);
        }
        else
        {
            ++iterator;
        }
    }

    // Remove the landscape, it's job is done!
    delete _landscape;
    _landscape = NULL;
}

////////////////////////////////////////////////////////////////////////////////
//
// Function    : recordTankDeath
//
// Description : Everytime a tank dies this function is called. It's used to 
//               determine when it's time to end the round.
//
////////////////////////////////////////////////////////////////////////////////
void
cGame::recordTankDeath 
(
)
{ 
    _numberOfTanksInPlay--; 

    if (_numberOfTanksInPlay < 2 && _gameState == ROUND_IN_ACTION)
    {
        // Only one tank left, start a 5 second countdown to the end of the 
        // round.
        _gameState = ROUND_FINISHING;
        _stateCountdown = 5.0f;
    }
}

////////////////////////////////////////////////////////////////////////////////
//
// Function    : addTank
//
// Description : Creates a new player (tank)
//
////////////////////////////////////////////////////////////////////////////////
void
cGame::addTank 
(
    int controller // Which controller will this player be using.
)
{
    _tank[_numberOfTanks] = new cTank (this, _numberOfTanks);
    
    // Ok, each tank has unique colours read from the settings file. Also give 
    // each tank a default name.
    switch (_numberOfTanks) 
    {
    case 0:
        _tank[0]->setColour
            (
                _settings.getFloat ("Colours", "Tank1red",   0.00f),
                _settings.getFloat ("Colours", "Tank1green", 0.00f),
                _settings.getFloat ("Colours", "Tank1blue",  0.00f)
            );
        
        _tank[0]->setName ("Player 1");
        break;
        
    case 1:
        _tank[1]->setColour
            (
                _settings.getFloat ("Colours", "Tank2red",   0.00f),
                _settings.getFloat ("Colours", "Tank2green", 0.00f),
                _settings.getFloat ("Colours", "Tank2blue",  0.00f)
            );
        
        _tank[1]->setName ("Player 2");
        break;
        
    case 2:
        _tank[2]->setColour
            (
                _settings.getFloat ("Colours", "Tank3red",   0.00f),
                _settings.getFloat ("Colours", "Tank3green", 0.00f),
                _settings.getFloat ("Colours", "Tank3blue",  0.00f)
            );
        
        _tank[2]->setName ("Player 3");
        break;
        
    case 3:
        _tank[3]->setColour
            (
                _settings.getFloat ("Colours", "Tank4red",   0.00f),
                _settings.getFloat ("Colours", "Tank4green", 0.00f),
                _settings.getFloat ("Colours", "Tank4blue",  0.00f)
            );
        
        _tank[3]->setName ("Player 4");
        break;
        
    case 4:
        _tank[4]->setColour
            (
                _settings.getFloat ("Colours", "Tank5red",   0.00f),
                _settings.getFloat ("Colours", "Tank5green", 0.00f),
                _settings.getFloat ("Colours", "Tank5blue",  0.00f)
            );
        
        _tank[4]->setName ("Player 5");
        break;
        
    case 5:
        _tank[5]->setColour
            (
                _settings.getFloat ("Colours", "Tank6red",   0.00f),
                _settings.getFloat ("Colours", "Tank6green", 0.00f),
                _settings.getFloat ("Colours", "Tank6blue",  0.00f)
            );
        
        _tank[5]->setName ("Player 6");
        break;
        
    case 6:
        _tank[6]->setColour
            (
                _settings.getFloat ("Colours", "Tank7red",   0.00f),
                _settings.getFloat ("Colours", "Tank7green", 0.00f),
                _settings.getFloat ("Colours", "Tank7blue",  0.00f)
            );
        
        _tank[6]->setName ("Player 7");
        break;
        
    case 7:
        _tank[7]->setColour
            (
                _settings.getFloat ("Colours", "Tank8red",   0.00f),
                _settings.getFloat ("Colours", "Tank8green", 0.00f),
                _settings.getFloat ("Colours", "Tank8blue",  0.00f)
            );
        
        _tank[7]->setName ("Player 8");
        break;
    }
    
    _tank[_numberOfTanks]->setController (controller);

    addEntity (_tank[_numberOfTanks]);

    _numberOfTanks++;
}

////////////////////////////////////////////////////////////////////////////////
//
// Function    : removeAllTanks
//
// Description : Kills all the tanks and removes the entityList. Only do this 
//               when the entity list is empty (except for the tanks.)
//
////////////////////////////////////////////////////////////////////////////////
void
cGame::removeAllTanks
(
)
{
    for (int i = 0 ; i < 8; i++) 
    {
        if (_tank[i] != NULL)
        {
            delete _tank[i];
            
            _tank[i] = NULL;
        }
    }

    _numberOfTanks = 0;

    _entityList.clear ();
}
