////////////////////////////////////////////////////////////////////////////////
//
//               Groundfire
//
////////////////////////////////////////////////////////////////////////////////
//
// Copyright (c) 2004, Tom Russell (tom@groundfire.net)
//
// This file is part of the Groundfire project, distributed under the MIT 
// license. See the file 'COPYING', included with this distribution, for a copy
// of the full MIT licence.
//
////////////////////////////////////////////////////////////////////////////////
//
//   File name : playermenu.cc
//
//          By : Tom Russell
//
//        Date : 26-May-03
//
// Description : Handles the player selection menu
//
//
//
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Includes
////////////////////////////////////////////////////////////////////////////////
#include "playermenu.hh"
#include "tank.hh"
#include "controls.hh"

////////////////////////////////////////////////////////////////////////////////
// Public Member Functions
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
//
// Function    : cPlayerMenu
//
// Description : Constructor
//
////////////////////////////////////////////////////////////////////////////////
cPlayerMenu::cPlayerMenu 
(
    cGame * game
)
: cMenu (game)
{
    // Create the number of rounds selector and populate it with the possible 
    // options.
    _numberOfRounds = new cSelector (this, 2.0f, -4.0f, 2.0f, 0.7f);

    _numberOfRounds->addOption ("5");
    _numberOfRounds->addOption ("10");
    _numberOfRounds->addOption ("15");
    _numberOfRounds->addOption ("20");
    _numberOfRounds->addOption ("25");
    _numberOfRounds->addOption ("30");
    _numberOfRounds->addOption ("35");
    _numberOfRounds->addOption ("40");
    _numberOfRounds->addOption ("45");
    _numberOfRounds->addOption ("50");

    _startButton = new cTextButton (this, 0.0f, -5.0f, 0.7f, "Start!");
    _backButton  = new cTextButton (this, 0.0f, -6.0f, 0.7f, "Back");

    _tanksJoined = 0;

    _startButton->enable (false);
}

////////////////////////////////////////////////////////////////////////////////
//
// Function    : ~cPlayerMenu
//
// Description : Destructor
//
////////////////////////////////////////////////////////////////////////////////
cPlayerMenu::~cPlayerMenu
(
)
{
    delete _numberOfRounds;
    delete _startButton;
    delete _backButton;
}

////////////////////////////////////////////////////////////////////////////////
//
// Function    : update
//
// Description : Updates the menu
//
////////////////////////////////////////////////////////////////////////////////
enumGameState
cPlayerMenu::update
(
    double time
)
{
    // Get a pointer to the tanks array for easy access later
    cTank ** tanks = _game->getTanks ();

    updateBackground (time);
    
    _numberOfRounds->update ();

    if (_startButton->update ())
    {
        // We're start the game! Set the number of rounds
        int rounds = (1 + _numberOfRounds->getOption ()) * 5;

        _game->setNumOfRounds (rounds);

        return (ROUND_STARTING);
    }

    if (_backButton->update ()) 
    {
        // Change of mind, go back to the title screen
        _game->removeAllTanks ();

        return (MAIN_MENU);
    }
    
    // Check each controller to see if fire has been pressed.
    for (int i = 0; i < (_game->getInterface ())->numOfControllers (); i++) 
    {
        if ((_game->getControls ())->getCommand (i, CMD_FIRE))
        {
            bool found = false;
            
            // Make sure this controller has not already been assigned to a tank
            for (int j = 0 ; j < _tanksJoined; j++) 
            {                    
                if (tanks[j]->getController () == i) 
                {
                    found = true;
                    break;
                }
            }
            
            if (!found)
            {
                // This controller is not being used so we can add
                // a new tank! (assuming there's one available.)
                if (_tanksJoined < 8) 
                {                      
                    _game->addTank (i);
                    _tanksJoined++;

                    // Once 2 or more tanks have joined, we can allow the game 
                    // to start by enabling the start button.
                    if (_tanksJoined > 1) 
                    {
                        _startButton->enable (true);
                    }
                }
            }
        }
    }
    
    return (CURRENT_STATE);
}

////////////////////////////////////////////////////////////////////////////////
//
// Function    : draw
//
// Description : Draw the player menu
//
////////////////////////////////////////////////////////////////////////////////
void
cPlayerMenu::draw
(
)
{
    drawBackground ();
    
    // Draw the menu title
    _font->setShadow (true);
    _font->setSize   (0.6f, 0.6f, 0.5f);
    _font->setColour (1.0f, 1.0f, 1.0f);

    _font->printCentredAt (0.0f, 6.5f, "Select Players");

    _font->setSize   (0.4f, 0.4f, 0.35f);
    _font->printCentredAt (0.0f, 5.5f,
                           "Press 'Fire' Button on any Controller to join in!");

    // Draw the transparent boxes for each tank entry

    glLoadIdentity ();

    glEnable (GL_BLEND);
  
    glColor4f (0.0f, 0.0f, 0.0f, 0.5f);

    cTank ** tanks = _game->getTanks ();

    glBegin (GL_QUADS);

    for (int i = 0; i < 4; i++)
    {
        if (tanks[i] != NULL)
        {
            glVertex3f   (-9.0f, 3.7f - i * 1.6f, 0.0f);
            glVertex3f   (-0.2f, 3.7f - i * 1.6f, 0.0f);
            glVertex3f   (-0.2f, 2.5f - i * 1.6f, 0.0f);
            glVertex3f   (-9.0f, 2.5f - i * 1.6f, 0.0f);
        }

        if (tanks[i + 4] != NULL)
        {
            glVertex3f   ( 0.2f, 3.7f - i * 1.6f, 0.0f);
            glVertex3f   ( 9.0f, 3.7f - i * 1.6f, 0.0f);
            glVertex3f   ( 9.0f, 2.5f - i * 1.6f, 0.0f);
            glVertex3f   ( 0.2f, 2.5f - i * 1.6f, 0.0f);
        }
    }

    // Draw the transparent boxes for the other options

    glVertex3f   (-7.0f, -6.6f, 0.0f);
    glVertex3f   ( 7.0f, -6.6f, 0.0f);
    glVertex3f   ( 7.0f, -3.4f, 0.0f);
    glVertex3f   (-7.0f, -3.4f, 0.0f);

    glColor4f (0.6f, 0.3f, 0.0f, 0.5f);

    glVertex3f   (-4.0f, -4.4f, 0.0f);
    glVertex3f   ( 4.0f, -4.4f, 0.0f);
    glVertex3f   ( 4.0f, -3.6f, 0.0f);
    glVertex3f   (-4.0f, -3.6f, 0.0f);

    glVertex3f   (-4.0f, -5.4f, 0.0f);
    glVertex3f   ( 4.0f, -5.4f, 0.0f);
    glVertex3f   ( 4.0f, -4.6f, 0.0f);
    glVertex3f   (-4.0f, -4.6f, 0.0f);

    glVertex3f   (-4.0f, -6.4f, 0.0f);
    glVertex3f   ( 4.0f, -6.4f, 0.0f);
    glVertex3f   ( 4.0f, -5.6f, 0.0f);
    glVertex3f   (-4.0f, -5.6f, 0.0f);

    glEnd ();

    glDisable (GL_BLEND);

    // Draw the tank graphic for each tank

    glBegin (GL_QUADS);

    float r, g, b;

    for (int i = 0; i < 4; i++)
    {
        if (tanks[i] != NULL)
        {
            tanks[i]->getColour (r, g, b);
            glColor3f (r, g, b);

            glVertex3f   (-8.3f, 3.4f - i * 1.6f, 0.0f);
            glVertex3f   (-7.7f, 3.4f - i * 1.6f, 0.0f);
            glVertex3f   (-7.4f, 2.8f - i * 1.6f, 0.0f);
            glVertex3f   (-8.6f, 2.8f - i * 1.6f, 0.0f);
        }

        if (tanks[i + 4] != NULL)
        {
            tanks[i + 4]->getColour (r, g, b);
            glColor3f (r, g, b);

            glVertex3f   ( 0.9f, 3.4f - i * 1.6f, 0.0f);
            glVertex3f   ( 1.5f, 3.4f - i * 1.6f, 0.0f);
            glVertex3f   ( 1.8f, 2.8f - i * 1.6f, 0.0f);
            glVertex3f   ( 0.6f, 2.8f - i * 1.6f, 0.0f);
        }
    }

    glEnd ();

    _font->setSize   (0.7f, 0.7f, 0.6f);
            
    _font->setColour (1.0f, 1.0f, 1.0f);
    _font->printCentredAt (-2.0f, -4.35f, "Rounds :");
    _font->setShadow (false);

    // Write the name of each tank

    _font->setSize   (0.5f, 0.5f, 0.4f);
    _font->setColour (0.6f, 0.6f, 0.6f);

    char * tankName;

    for (int i = 0; i < 4; i++) 
    {
        if (tanks[i] != NULL)
        {  
            tankName = tanks[i]->getName ();

            _font->printf (-7.0f, 2.8f - i * 1.6f, "%s", tankName);
        } 

        if (tanks[i + 4] != NULL)
        {
            tankName = tanks[i + 4]->getName ();

            _font->printf (2.2f, 2.8f - i * 1.6f, "%s", tankName);
        }
    }

    _numberOfRounds->draw ();
    _startButton->draw ();
    _backButton->draw ();
}
