////////////////////////////////////////////////////////////////////////////////
//
//               Groundfire
//
////////////////////////////////////////////////////////////////////////////////
//
// Copyright (c) 2004, Tom Russell (tom@groundfire.net)
//
// This file is part of the Groundfire project, distributed under the MIT 
// license. See the file 'COPYING', included with this distribution, for a copy
// of the full MIT licence.
//
////////////////////////////////////////////////////////////////////////////////
//
//   File name : shell.cc
//
//          By : Tom Russell
//
//        Date : 08-Sep-02
//
// Description : Handles the shell projectiles (i.e. standard gun projectile / 
//               nukes)
//
//
//
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Includes
////////////////////////////////////////////////////////////////////////////////
#include "shell.hh"
#include "game.hh"
#include "tank.hh"
#include "blast.hh"
#include "common.hh"
#include "soundentity.hh"

////////////////////////////////////////////////////////////////////////////////
// Public Member Functions
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
//
// Function    : cShell
//
// Description : Constructor
//
////////////////////////////////////////////////////////////////////////////////
cShell::cShell
(
    cGame * game,
    cTank * tank,
    float x,
    float y,
    float xVel,
    float yVel,
    float size,    // The size of the blast created when the shell explodes
    float damage,  // Amount of damage a direct hit from the shell will do 
    bool  whiteOut // Will the shell cause a white flash when it explodes? 
)
: cEntity (game),
  _tank (tank),
  _xVel (xVel),
  _yVel (yVel),
  _size (size),
  _damage (damage),
  _whiteOut (whiteOut)
{
    _x = x;
    _y = y;

    // Create a new trail object to leave behind the shell
    _trail = new cTrail (game, _x, _y);
    _game->addEntity (_trail);
}

////////////////////////////////////////////////////////////////////////////////
//
// Function    : ~cShell
//
// Description : Destructor
//
////////////////////////////////////////////////////////////////////////////////
cShell::~cShell
(
)
{
}

////////////////////////////////////////////////////////////////////////////////
//
// Function    : draw
//
// Description : Draw the shell.
//
////////////////////////////////////////////////////////////////////////////////
void
cShell::draw
(
    void
)
{
    glLoadIdentity ();

    glTranslatef (_x, _y, -6.0f);

    glColor3f (1.0f, 1.0f, 1.0f);

    // Shells are currently drawn as tiny triangles (they look like dots)
    glBegin (GL_TRIANGLES);
    glVertex3f ( 0.00f,  0.018f, 0.0f);
    glVertex3f ( 0.03f, -0.018f, 0.0f);
    glVertex3f (-0.03f, -0.018f, 0.0f);
    glEnd ();
}

////////////////////////////////////////////////////////////////////////////////
//
// Function    : update
//
// Description : Update the shell
//
////////////////////////////////////////////////////////////////////////////////
bool
cShell::update
(
    float time
)
{
    float oldX = _x;
    float oldY = _y;

    float collisionX;
    float collisionY;
    
    // Update the shell position and velocity.
    _x    += time * _xVel;
    _y    += time * _yVel;
    _yVel -= time * 10.0f; // The gravity effect

    // If the shell has left the side of the screen.
    if (_x >  (_game->getLandscape ())->getLandscapeWidth () ||
        _x < -(_game->getLandscape ())->getLandscapeWidth ()) 
    {

        // Lay a final piece of trail and stop laying trail.
        _trail->layTrail (_x, _y);
        _trail->setInactive ();

        // shell has left the side of the screen so destroy it
        delete this;
        return (false);
    }

    // Have we hit the ground?
    if ((_game->getLandscape ())->groundCollision (oldX, oldY, _x, _y, 
                                                   &collisionX, 
                                                   &collisionY))
    {       
        _trail->layTrail (collisionX, collisionY);

        explode (collisionX, collisionY, -1);

        return (false);
    }

    _trail->layTrail (_x, _y);

    // Check for collision with a tank
    cTank ** tanks = _game->getTanks ();

    for (int i = 0; i < 8 && tanks[i] != NULL; i++)
    {
        if (tanks[i]->pointInTank (_x, _y))
        {
            explode (_x, _y, i);

            return (false);
        }
    }
    
    
    return (true);
}

////////////////////////////////////////////////////////////////////////////////
//
// Function    : explode
//
// Description : Explode the shell at the coordinates provided
//
////////////////////////////////////////////////////////////////////////////////
void
cShell::explode
(
    float x, 
    float y,
    int   hitTank // The tank to deal full damage to (i.e. the tank we hit)
)
{ 
    // Make a hole in the landscape
    (_game->getLandscape ())->makeHole (x, y, _size); 

    // Create a blast graphic
    cBlast * blast = new cBlast (_game, x, y, _size, 0.8f, _whiteOut);
    _game->addEntity (blast);

    // Shells that produce a white out play the 'Nuke' sound, otherwise just 
    // the normal small explosion sound.
    cSoundEntity * blastSound;
    if (_whiteOut)
    {
        blastSound = new cSoundEntity (_game, 7, false);   
    }
    else
    {
        blastSound = new cSoundEntity (_game, 1, false);   
    }
    _game->addEntity (blastSound);
    
    _trail->setInactive ();
    
    cTank ** tanks = _game->getTanks ();
    
    float tankX;
    float tankY;
    
    for (int i = 0; i < 8 && tanks[i] != NULL; i++)
    {
        if (i == hitTank)
        {
            // Do maximum damage to the hit tank
            if (tanks[i]->doDamage (_damage))
            {
                // The tank was destroyed, credit the kill to the owner of this 
                // shell.
                _tank->defeated (tanks[i]);
            }
        }
        else
        {
            float hitRange = tanks[i]->getCenter (&tankX, &tankY);
            
            float squaredDistance = sqr (tankX - x) + sqr (tankY - y);
                
            float maxDistance = sqr (_size + hitRange);

            if (squaredDistance < maxDistance) 
            {
                // Tank was in range so it will take some damage.
                if (tanks[i]->doDamage 
                        (_damage * (1 - (squaredDistance / maxDistance))))
                {
                    // The tank was destroyed, credit the kill to the owner
                    // of this shell.
                    _tank->defeated (tanks[i]);
                }
            }
        }
    }

    delete this;
}
