////////////////////////////////////////////////////////////////////////////////
//
//               Groundfire
//
////////////////////////////////////////////////////////////////////////////////
//
// Copyright (c) 2004, Tom Russell (tom@groundfire.net)
//
// This file is part of the Groundfire project, distributed under the MIT 
// license. See the file 'COPYING', included with this distribution, for a copy
// of the full MIT licence.
//
////////////////////////////////////////////////////////////////////////////////
//
//   File name : shopmenu.cc
//
//          By : Tom Russell
//
//        Date : 01-Jul-03
//
// Description : Handles the shopping menu which appears between rounds
//
//
//
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Includes
////////////////////////////////////////////////////////////////////////////////
#include "shopmenu.hh"

#include "tank.hh"
#include "weapon.hh"
#include "controls.hh"

////////////////////////////////////////////////////////////////////////////////
// Public Member Functions
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
//
// Function    : cShopMenu
//
// Description : Constructor
//
////////////////////////////////////////////////////////////////////////////////
cShopMenu::cShopMenu
(
    cGame * game
)
: cMenu (game)
{
    for (int i = 0; i < 8; i++) 
    {
        _tankSelectPos[i]   = 0;
        _tankSelectDelay[i] = 0.0f;
        _tankDone[i]        = false;
    }

    // 'linelit' is to prevent us drawing too many transparent bars across one 
    // option.
    for (int i = 0; i < 10; i++) 
    {
        _lineLit[i] = false;
    }
}

////////////////////////////////////////////////////////////////////////////////
//
// Function    : ~cShopMenu
//
// Description : Destructor
//
////////////////////////////////////////////////////////////////////////////////
cShopMenu::~cShopMenu
(
)
{
}

////////////////////////////////////////////////////////////////////////////////
//
// Function    : update
//
// Description : Update the shopping menu
//
////////////////////////////////////////////////////////////////////////////////
enumGameState
cShopMenu::update
(
    double time
)
{
    updateBackground (time);

    cTank ** tanks = _game->getTanks ();

    bool stillTanksInShop = false;

    for (int i = 0; i < 8; i++) 
    {
        if (tanks[i] != NULL && _tankDone[i] == false)
        {
            stillTanksInShop = true;

            // Slow down the rate at which the user can scroll down the list by
            // pausing for a short delay after every move
            if (_tankSelectDelay[i] < 0.0f) 
            {
                if (_game->getControls ()->getCommand (
                        tanks[i]->getController (), CMD_GUNUP))
                {
                    _tankSelectPos[i]--;

                    if (_tankSelectPos[i] == -1) 
                    {
                        _tankSelectPos[i] = 10;
                    }

                    _tankSelectDelay[i] = 0.2f;
                }
                else if (_game->getControls ()->getCommand (
                             tanks[i]->getController (), CMD_GUNDOWN))
                {
                    _tankSelectPos[i]++;
                    
                    if (_tankSelectPos[i] == 11) 
                    {
                        _tankSelectPos[i] = 0;
                    }

                    _tankSelectDelay[i] = 0.2f;
                }
                else if (_game->getControls ()->getCommand (
                             tanks[i]->getController (), CMD_FIRE))
                {
                    // This tank is trying to purchase a weapon!

                    switch (_tankSelectPos[i])
                    {
                    case 0: // Machine Gun
                        break;

                    case 1: // Jump Jets
                        if (tanks[i]->_money >= 50) // Check funds first
                        {
                            tanks[i]->_money -= 50;
                            tanks[i]->_totalFuel += 1.0f;
                        }
                        break;

                    case 2: // Mirvs
                        break;
                        
                    case 3: // Missiles
                        if (   tanks[i]->_money
                               >= tanks[i]->_weapon[2]->getCost ())//check funds
                        {
                            // missiles come in packs of 5
                            tanks[i]->_money -= 
                                tanks[i]->_weapon[2]->getCost ();
                            tanks[i]->_weapon[2]->addAmount (5); 
                        }
                        break;
                        
                    case 4: // Nukes
                        if (   tanks[i]->_money
                               >= tanks[i]->_weapon[1]->getCost ())//check funds
                        {
                            tanks[i]->_money -=
                                tanks[i]->_weapon[1]->getCost ();
                            tanks[i]->_weapon[1]->addAmount (1);
                        }
                        break;
                        
                        // These aren't implemented yet!
                    case 5: // Rolling Mines
                    case 6: // Airstrike
                    case 7: // Death's Head
                    case 8: // Hover Coil
                    case 9: // Corbomite
                        break;

                        // We've finished 
                    case 10:
                        _tankDone[i] = true;
                        break;
                        
                    default:
                        break;
                    }

                    // The delay here is quite important. We don't want anyone 
                    // to accidentally buy multiple times
                    _tankSelectDelay[i] = 0.2f;    
                }
            }
            else
            {
                _tankSelectDelay[i] -= time;
            }

            _lineLit[_tankSelectPos[i]] = true;
        }
    }

    // Has everyone finished shopping?
    if (stillTanksInShop)
    {
        return (CURRENT_STATE);
    }
    else
    {
        return (ROUND_STARTING);
    }
}

////////////////////////////////////////////////////////////////////////////////
//
// Function    : draw
//
// Description : Draw the menu
//
////////////////////////////////////////////////////////////////////////////////
void
cShopMenu::draw
(
)
{
    drawBackground ();

    glEnable (GL_BLEND);
  
    glColor4f (0.6f, 0.3f, 0.0f, 0.5f);

    cTank ** tanks = _game->getTanks ();

    for (int i = 0; i < 8; i++)
    {
        if (tanks[i] != NULL && _tankDone[i] == false)
        {
            glBegin (GL_QUADS);
            
            glColor4f (0.3f, 0.1f, 0.1f, 0.5f);

            // Draw the vertical bars for each tank. The tabs showing the 
            // current funds appear at either the top or bottom on alternate 
            // tanks.

            if (i % 2 == 0)
            {
                glVertex3f   (-9.4f + (i * 1.5f),  5.6f, 0.0f);
                glVertex3f   (-9.4f + (i * 1.5f),  4.9f, 0.0f);
                glVertex3f   (-6.6f + (i * 1.5f),  4.9f, 0.0f);
                glVertex3f   (-6.6f + (i * 1.5f),  5.6f, 0.0f);

                glVertex3f   (-9.4f + (i * 1.5f), -5.3f, 0.0f);
                glVertex3f   (-9.4f + (i * 1.5f),  4.9f, 0.0f);
                glVertex3f   (-8.1f + (i * 1.5f),  4.9f, 0.0f);
                glVertex3f   (-8.1f + (i * 1.5f), -5.3f, 0.0f);

            }
            else
            {
                glVertex3f   (-10.9f + (i * 1.5f), -5.5f, 0.0f);
                glVertex3f   (-10.9f + (i * 1.5f), -6.2f, 0.0f);
                glVertex3f   (-8.1f  + (i * 1.5f), -6.2f, 0.0f);
                glVertex3f   (-8.1f  + (i * 1.5f), -5.5f, 0.0f);

                glVertex3f   (-9.4f + (i * 1.5f), -5.5f, 0.0f);
                glVertex3f   (-9.4f + (i * 1.5f),  4.7f, 0.0f);
                glVertex3f   (-8.1f + (i * 1.5f),  4.7f, 0.0f);
                glVertex3f   (-8.1f + (i * 1.5f), -5.5f, 0.0f);
            }
            
            // if the weapons line the tank is on is already lit, don't light 
            // it again. This prevents it becoming saturated.
            if (_lineLit[_tankSelectPos[i]] == true)
            {
                float v = _tankSelectPos[i] * 0.8f;

                glColor4f (1.0f, 1.0f, 1.0f, 0.1f);
                glVertex3f   (-9.4f, 3.8f - v, 0.0f);
                glVertex3f   (-9.4f, 4.6f - v, 0.0f);
                glVertex3f   ( 9.4f, 4.6f - v, 0.0f);
                glVertex3f   ( 9.4f, 3.8f - v, 0.0f);

                _lineLit[_tankSelectPos[i]] = false;
            }
            
            glEnd ();
        }
    }

    glDisable (GL_BLEND);

    _font->setSize (0.3f, 0.3f, 0.25f);
    _font->setColour (1.0f, 1.0f, 1.0f);

    for (int i = 0; i < 8; i++) 
    {
        if (tanks[i] != NULL && _tankDone[i] == false)
        {
            // Draw a graphic of each tank
            glColor3f (tanks[i]->_colourRed, 
                       tanks[i]->_colourGreen, 
                       tanks[i]->_colourBlue);

            float v = _tankSelectPos[i] * 0.8f;

            glLoadIdentity ();

            glBegin (GL_QUADS);

            glVertex3f   (-9.35f + (i * 1.5f), 3.9f - v, 0.0f);
            glVertex3f   (-9.05f + (i * 1.5f), 4.5f - v, 0.0f);
            glVertex3f   (-8.45f + (i * 1.5f), 4.5f - v, 0.0f);
            glVertex3f   (-8.15f + (i * 1.5f), 3.9f - v, 0.0f);

            glEnd ();    

            // Below the tank graphic, draw an indicator of the current amount 
            // of the weapon we are currently lined up with. This is either a 
            // bar (for jumpjets) or a number representing the current 
            // inventory (e.g. nukes)

            switch (_tankSelectPos[i])
            {
            case 0: // Machine Gun
                break;

            case 1: // Jump Jets
                drawBars (-9.4f + (i * 1.5f), 3.7f - v, tanks[i]->_totalFuel);
                break;

            case 2: // Mirvs
                break;

            case 3: // Missiles
                _font->printCentredAt (-8.75 + (i * 1.5f), 3.5f - v, "x%d", 
                                       tanks[i]->_weapon[2]->getAmmo ());
                break;

            case 4: // Nukes
                _font->printCentredAt (-8.75 + (i * 1.5f), 3.5f - v, "x%d", 
                                       tanks[i]->_weapon[1]->getAmmo ());
                break;

            case 5: // Rolling Mines
            case 6: // Airstrike
            case 7: // Death's Head
            case 8: // Hover Coil
            case 9: // Corbomite
                break;

            default:
                break;
            }
        }
    }

    _font->setShadow (true);
    // Write the number of the round we're about to start.
    _font->setSize (0.6f, 0.6f, 0.5f);
    _font->setColour (1.0f, 1.0f, 1.0f);

    _font->printCentredAt (0.0f, 6.5f, "Round %d of %d", 
                           _game->getCurrentRound () + 1, 
                           _game->getNumOfRounds ());

    _font->setSize (0.4f, 0.4f, 0.3f);
    _font->setColour (0.9f, 0.9f, 0.9f);
    
    // Write the weapon names for each row.

    _font->printCentredAt (4.0f,  5.0f, "Cost");
    _font->printCentredAt (7.0f,  5.0f, "Item");

    _font->setColour (0.3f, 0.3f, 0.3f);
    _font->printCentredAt (7.0f,  4.0f, "Machine Gun");
    _font->printCentredAt (4.0f,  4.0f, "$50");
    _font->setColour (0.9f, 0.9f, 0.9f);
    _font->printCentredAt (7.0f,  3.2f, "Jump Jet");
    _font->printCentredAt (4.0f,  3.2f, "$50");
    _font->setColour (0.3f, 0.3f, 0.3f);
    _font->printCentredAt (7.0f,  2.4f, "Mirvs");
    _font->printCentredAt (4.0f,  2.4f, "$50");
    _font->setColour (0.9f, 0.9f, 0.9f);
    _font->printCentredAt (7.0f,  1.6f, "Missiles");
    _font->printCentredAt (4.0f,  1.6f, "$50");
    _font->printCentredAt (7.0f,  0.8f, "Nukes");
    _font->printCentredAt (4.0f,  0.8f, "$50");
    _font->setColour (0.3f, 0.3f, 0.3f);
    _font->printCentredAt (7.0f,  0.0f, "Rolling Mines");
    _font->printCentredAt (4.0f,  0.0f, "$50");
    _font->printCentredAt (7.0f, -0.8f, "Airstrike");
    _font->printCentredAt (4.0f, -0.8f, "$100");
    _font->printCentredAt (7.0f, -1.6f, "Death's Head");
    _font->printCentredAt (4.0f, -1.6f, "$200");
    _font->printCentredAt (7.0f, -2.4f, "Hover Coil");
    _font->printCentredAt (4.0f, -2.4f, "$150");
    _font->printCentredAt (7.0f, -3.2f, "Corbomite");
    _font->printCentredAt (4.0f, -3.2f, "$20");

    _font->setColour (0.9f, 0.9f, 0.9f);
    _font->printCentredAt (7.0f, -4.0f, "Done!");

    _font->setSize (0.35f, 0.35f, 0.275f);

    // Write the amount of money everybody has
    for (int i = 0; i < 8; i++)
    {
        if (tanks[i] != NULL && _tankDone[i] == false)
        {
            if (i % 2 == 0) 
            {
                _font->printCentredAt (-8.0f + (i * 1.5f), 5.1f, 
                               "$%d", tanks[i]->_money);
            }
            else
            {
                _font->printCentredAt (-9.5f + (i * 1.5f), -6.0f, 
                               "$%d", tanks[i]->_money);    
            }
        }
    }
    _font->setShadow (false);
}

////////////////////////////////////////////////////////////////////////////////
//
// Function    : drawBars
//
// Description : Draw the bars representing inventory (of things like jumpjet 
//               fuel)
//
////////////////////////////////////////////////////////////////////////////////
void
cShopMenu::drawBars
(
    float x,
    float y,
    float numberOfBars
)
{
    glColor3f (1.0f, 1.0f, 1.0f);

    for (int i = 0; numberOfBars > 0.0f; i++)
    {
        float lengthOfBar;

        if (numberOfBars > 1.0f)
        {
            lengthOfBar = 1.3f;
        }
        else
        {
            lengthOfBar = 1.3f * numberOfBars;
        }
        numberOfBars -= 1.0f;

        glBegin (GL_QUADS);

        glVertex3f   (x,               y - (0.2f * i) + 0.15f, 0.0f);
        glVertex3f   (x,               y - (0.2f * i),         0.0f);
        glVertex3f   (x + lengthOfBar, y - (0.2f * i),         0.0f);
        glVertex3f   (x + lengthOfBar, y - (0.2f * i) + 0.15f, 0.0f);

        glEnd ();  
    } 
}
